# -*- coding: utf-8 -*-

"""
Ce programme propose de simuler le modèle de Hardy-Weinberg en s'intéressant
à un gène avec 2 allèles A et a.
"""

# Permet d'importer un module pour générer des nombres pseudo-aléatoire.
import random

# Permet d'importer un module pour tracer des graphiques.
import matplotlib.pyplot as plt

# p est la fréquence de l'allèle A et q la fréquence de l'allèle a
p = float(input("Rentrer ici la fréquence p de l'allèle A : "))
q = 1 - p

# Création de 2 listes pour suivre l'évolution des fréquences des allèles
evo_p = [p]
evo_q = [q]

# Nombre de générations
generation = int(input("Rentrer ici le nombre de générations : "))
gen = generation

# Population à chaque génération
x = int(input("Rentrer ici le nombre d'individus par génération : "))

# Boucle pour simuler la reproduction des individus d’une population sur 
# le nombre de générations souhaité
j = 0
while j < generation:
    # Boucle pour générer une population de x individus avec des zygotes 
    # générés aléatoirement en suivant les probabilités p et q
    zygotes = []
    i = 0
    while i < x:
        # Génération du gamète 1
        if random.random() < p:
            gamete1="A"
        else:
            gamete1="a"
    
        # Génération du gamète 2
        if random.random() < p:
            gamete2="A"
        else:
            gamete2="a"

        # Formation du zygote et ajout à la population des zygotes
        zygotes.append(gamete1+gamete2)
        
        i += 1

    # Dénombrement des allèles A et a
    nbA = 0
    nba = 0
    for i in range(len(zygotes)):
        if zygotes[i] == "AA":
            nbA += 2
        elif zygotes[i] == "aa":
            nba += 2
        else:
            nbA += 1
            nba += 1

    # Nouvelles valeurs de p et q
    p = nbA/(nbA+nba)
    q = nba/(nbA+nba)

    # Sauvegarde des valeurs de p et q
    evo_p.append(p)
    evo_q.append(q)

    j += 1

# Présentation des résultats sous forme graphique
plt.plot(range(len(evo_p)), evo_p, label="p : Fréquence de l'allèle A")
plt.plot(range(len(evo_q)), evo_q, label="q : Fréquence de l'allèle a")
plt.legend()
plt.xlabel("Nombre de générations")
plt.ylabel("Fréquence des allèles")
plt.ylim(0, 1)
plt.title('Simulation du modèle de Hardy-Weinberg sur ' + str(gen) +
          ' générations avec ' + str(x) + ' individus par génération')
plt.show()
